import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_route.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';

final List<String> _randomIndicative = [
  'exp1',
  'exp2',
  'exp3',
  'exp4',
  'exp5',
  'exp6',
];

final List<String> _randomPronouns = [
  'pron1',
  'pron2',
  'pron3',
  'pron4',
  'pron5'
  'pron6',
];

final Finder nextPersonKeyFinder = find.byKey(Key('nextPersonKey'));
final Finder expectedVerbFinder = find.byKey(Key('expectedVerb'));
final Finder actualVerbFinder = find.byKey(Key('actualVerb'));
final Finder statusIconFinder = find.byKey(Key('comparisonStatusIcon'));
final Finder iconFinder = find.byIcon(Icons.assignment_turned_in_outlined);
final Finder tappableIconFinder = find.byKey(Key('tappableStatus'));

void main() {
  testWidgets('Test the state of the verb', (WidgetTester tester) async {
    await tester.pumpWidget(ChangeNotifierProvider(
      create: (BuildContext context) {
        var model = VerbTestModel();
        model.initialise(_randomIndicative, _randomPronouns, 'f', 'infinitive',
            'translation');
        return model;
      },
      child: VerbIndicativeStudyRoute(StudyMode.writtenMode),
    ));

    expect(nextPersonKeyFinder, findsNothing);
    expect(expectedVerbFinder, findsOneWidget);
    Finder expectedVerbValueFinder = find.text('exp1');
    expect(expectedVerbValueFinder, findsOneWidget);
    expect(actualVerbFinder, findsOneWidget);
    expect(statusIconFinder, findsOneWidget);
    expect(iconFinder, findsOneWidget);
    await tester.enterText(actualVerbFinder, 's');
    await tester.pump();
    Finder wrongLetterFinder = find.text('s');
    expect(wrongLetterFinder, findsOneWidget);
    expect(tappableIconFinder, findsOneWidget);
    await tester.tap(tappableIconFinder);
    await tester.pump();
    testIconColor(tester, iconFinder, Colors.red);
    await tester.enterText(actualVerbFinder, 'exp');
    await tester.tap(tappableIconFinder);
    await tester.pump();
    testIconColor(tester, iconFinder, Colors.orange);
    await tester.enterText(actualVerbFinder, 'exp1');
    await tester.tap(tappableIconFinder);
    await tester.pump();
    testIconColor(tester, iconFinder, Colors.green);
    await tester.enterText(actualVerbFinder, 'exp1blbla');
    await tester.tap(tappableIconFinder);
    await tester.pump();
    testIconColor(tester, iconFinder, Colors.red);
    await tester.enterText(actualVerbFinder, _randomIndicative[0]);
    await tester.pump();
    expect(nextPersonKeyFinder, findsOneWidget);
    Finder secondPersonExpectedFinder = find.text(_randomIndicative[1]);
    expect(secondPersonExpectedFinder, findsNothing);
    await tester.tap(nextPersonKeyFinder);
    await tester.pump();
    expect(secondPersonExpectedFinder, findsOneWidget);
    Finder toTestButton = find.byKey(Key('write_forward_button'));
    expect(toTestButton, findsNothing);
    for (int i = 1; i < 5; i++) {
      await tester.enterText(actualVerbFinder, _randomIndicative[i]);
      await tester.pump();
      expect(nextPersonKeyFinder, findsOneWidget);
      await tester.tap(nextPersonKeyFinder);
      await tester.pump();
    }
    expect(toTestButton, findsOneWidget);
  });

  testWidgets('Test comparisonstatus in happy day circumstances',
      (WidgetTester tester) async {
    await tester.pumpWidget(ChangeNotifierProvider(
      create: (BuildContext context) {
        var model = VerbTestModel();
        model.initialise(_randomIndicative, _randomPronouns, 'f', 'infinitive',
            'translation');
        return model;
      },
      child: VerbIndicativeStudyRoute(StudyMode.writtenMode),
    ));
    expect(tappableIconFinder, findsOneWidget);
    await tester.pump();
    for (int i = 0; i < 5; i++) {
      testIconColor(tester, iconFinder, Colors.grey);
      for (int j = 1; j <= _randomIndicative[i].length; j++) {
        final String textToEnter = _randomIndicative[i].substring(0, j);
        await tester.enterText(actualVerbFinder, textToEnter);
        await tester.tap(tappableIconFinder);
        await tester.pump();
        if (textToEnter.length < _randomIndicative[i].length) {
          testIconColor(tester, iconFinder, Colors.orange);
        } else {
          testIconColor(tester, iconFinder, Colors.green);
        }
      }
      expect(nextPersonKeyFinder, findsOneWidget);
      await tester.tap(nextPersonKeyFinder);
      await tester.pump();
    }
  });
}

void testIconColor(WidgetTester tester, Finder iconFinder, Color color) {
  expect((tester.firstWidget(iconFinder) as Icon).color, color);
}
